//========================================================================================
//
//	File:				FWMnuBar.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWOS.hpp"

#ifndef FWMNUBAR_H
#include "FWMnuBar.h"
#endif

#ifndef FWPULLDM_H
#include "FWPullDM.h"
#endif

// ----- Foundation Includes -----

#ifndef FWINTSTR_H
#include "FWIntStr.h"
#endif

#ifndef FWSTRS_H
#include "FWStrs.h"
#endif

// ----- ODUtil Includes -----

#ifndef FWORDCOL_H
#include "FWOrdCol.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_ODPart_xh
#include <Part.xh>
#endif

#ifndef SOM_ODDraft_xh
#include <Draft.xh>
#endif

#ifndef SOM_ODStorageUnit_xh
#include <StorageU.xh>
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef SOM_ODWindowState_xh
#include <WinStat.xh>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__TOOLUTILS__)
#include <ToolUtils.h>
#endif

//========================================================================================
//	RunTime Info
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifdef FW_BUILD_MAC
#pragma segment fwmenu
#endif

//========================================================================================
//	class FW_CMenuBar
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::FW_CMenuBar
//----------------------------------------------------------------------------------------

FW_CMenuBar::FW_CMenuBar(Environment* ev, ODPart* thePart) :
	fODPart(thePart),
	fPullDownMenuList(NULL),
	fNextMenuID(FW_kFirstMenuID)
{
	fODMenuBar = fODPart->GetStorageUnit(ev)->GetSession(ev)->GetWindowState(ev)->CopyBaseMenuBar(ev);
	fPullDownMenuList = new FW_CPrivOrderedCollection;
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::~FW_CMenuBar
//----------------------------------------------------------------------------------------

FW_CMenuBar::~FW_CMenuBar()
{
	Environment* ev = somGetGlobalEnvironment();
	
	if (fPullDownMenuList)
		DeleteAll(ev);

	if (fODMenuBar)
		fODMenuBar->Release(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::AdoptMenuFirst
//----------------------------------------------------------------------------------------

void FW_CMenuBar::AdoptMenuFirst(Environment* ev, FW_CPullDownMenu* menu)
{
	FW_ASSERT(menu != NULL);
	FW_ASSERT(PrivFindMenuWithID(ev, menu->GetMenuID(ev)) == NULL); 	// Test that the menuID is not already used
	
	FW_CPullDownMenu* first = (FW_CPullDownMenu*)fPullDownMenuList->First();
	fPullDownMenuList->AddFirst(menu);
	
	ODMenuID beforeID = first == NULL ? -1 : first->GetMenuID(ev);	
	menu->PrivAttachedToMenuBar(ev, this, beforeID);
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::AdoptMenuLast
//----------------------------------------------------------------------------------------

void FW_CMenuBar::AdoptMenuLast(Environment* ev, FW_CPullDownMenu* menu)
{
	FW_ASSERT(menu != NULL);
	FW_ASSERT(PrivFindMenuWithID(ev, menu->GetMenuID(ev)) == NULL);	// Test that the menuID is not already used
	
	fPullDownMenuList->AddLast(menu);	
	menu->PrivAttachedToMenuBar(ev, this, -1);
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::AdoptMenuBefore
//----------------------------------------------------------------------------------------

void FW_CMenuBar::AdoptMenuBefore(Environment* ev, FW_CPullDownMenu* menu, FW_CPullDownMenu* beforeMenu)
{
	FW_ASSERT(menu != NULL);
	FW_ASSERT(PrivFindMenuWithID(ev, menu->GetMenuID(ev)) == NULL);	// Test that the menuID is not already used

	if (beforeMenu == NULL)
		AdoptMenuFirst(ev, menu);
	else
	{
		fPullDownMenuList->AddBefore(beforeMenu, menu);
		menu->PrivAttachedToMenuBar(ev, this, beforeMenu->GetMenuID(ev));
	}
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::AdoptMenuAfter
//----------------------------------------------------------------------------------------

void FW_CMenuBar::AdoptMenuAfter(Environment* ev, FW_CPullDownMenu* menu, FW_CPullDownMenu* afterMenu)
{
	FW_ASSERT(menu != NULL);
	FW_ASSERT(PrivFindMenuWithID(ev, menu->GetMenuID(ev)) == NULL);	// Test that the menuID is not already used

	if (afterMenu == NULL)
		AdoptMenuLast(ev, menu);
	else
	{
		FW_CPullDownMenu* next = (FW_CPullDownMenu*)fPullDownMenuList->After(afterMenu);
		fPullDownMenuList->AddAfter(afterMenu, menu);
		ODMenuID beforeID = next == NULL ? -1 : next->GetMenuID(ev);	
		menu->PrivAttachedToMenuBar(ev, this, beforeID);
	}
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::DeleteMenu
//----------------------------------------------------------------------------------------

void FW_CMenuBar::DeleteMenu(Environment* ev, FW_CPullDownMenu* menuToDelete)
{
	FW_ASSERT(menuToDelete != NULL);

	DetachMenu(ev, menuToDelete);
	delete menuToDelete;
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::DeleteAll
//----------------------------------------------------------------------------------------

void FW_CMenuBar::DeleteAll(Environment* ev)
{
	FW_CPullDownMenu *menu = (FW_CPullDownMenu*)fPullDownMenuList->First();
	while (menu != NULL)
	{
		DeleteMenu(ev, menu);
		menu = (FW_CPullDownMenu*)fPullDownMenuList->First();
	}
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::RemoveItem
//----------------------------------------------------------------------------------------

void FW_CMenuBar::RemoveItem(Environment* ev, ODCommandID commandID)
{
	FW_CMenuItem* item = this->GetMenuItemFromCommand(ev, commandID);
	FW_ASSERT(item);
	
	FW_CPullDownMenu* menu = item->GetOwnerMenu(ev);
	menu->RemoveItem(ev, item->GetIndex(ev));
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::DetachMenu
//----------------------------------------------------------------------------------------

void FW_CMenuBar::DetachMenu(Environment* ev, FW_CPullDownMenu* menuToDetach)
{
	FW_ASSERT(menuToDetach != NULL);
	
	fODMenuBar->RemoveMenu(ev, menuToDetach->GetMenuID(ev));
	
	fPullDownMenuList->Remove(menuToDetach);
	menuToDetach->PrivDetachedFromMenuBar(ev, this);
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::GetMenuKey
//----------------------------------------------------------------------------------------

FW_MenuKey FW_CMenuBar::GetMenuKey(Environment* ev, ODCommandID cmdNumber) const
{
	FW_CMenuItem* menuItem = GetMenuItemFromCommand(ev, cmdNumber);
	return menuItem->GetMenuKey(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::AcquireODMenuBar
//----------------------------------------------------------------------------------------

ODMenuBar* FW_CMenuBar::AcquireODMenuBar(Environment* ev) const
{
#if FW_OPENDOC_VERSION >= FW_OPENDOC_DR3
	fODMenuBar->Acquire(ev);
#else
	fODMenuBar->IncrementRefCount(ev);
#endif
	return fODMenuBar;
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::PrivFindMenuWithID
//----------------------------------------------------------------------------------------

FW_CPullDownMenu* FW_CMenuBar::PrivFindMenuWithID(Environment* ev, ODMenuID menuID) const
{
	// ----- Look the top level first -----
	FW_COrderedCollectionIterator ite(fPullDownMenuList);
	FW_CPullDownMenu* pullDownMenu;
	for (pullDownMenu = (FW_CPullDownMenu*)ite.First(); ite.IsNotComplete(); pullDownMenu = (FW_CPullDownMenu*)ite.Next())
	{
		if (pullDownMenu->GetMenuID(ev) == menuID)
			return pullDownMenu;
	}
	
	// ----- Then Look sub menus -----
	for (pullDownMenu = (FW_CPullDownMenu*)ite.First(); ite.IsNotComplete(); pullDownMenu = (FW_CPullDownMenu*)ite.Next())
	{
		FW_CPullDownMenu* menu = pullDownMenu->PrivFindMenuWithID(ev, menuID);
		if (menu)
			return menu;
	}
	
	return NULL;
}

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	FW_CMenuBar::MacEnableMenuBar
//----------------------------------------------------------------------------------------
//	This is specific for modal dialogs on the mac

void FW_CMenuBar::MacEnableMenuBar(Environment* ev, FW_Boolean enable)
{
	// ----- Enable first OpenDoc Menus -----
	fODMenuBar->EnableCommand(ev, kODCommandEditMenu, enable);
	fODMenuBar->EnableCommand(ev, kODCommandDocumentMenu, enable);

	// ----- Then custom menus -----
	FW_COrderedCollectionIterator ite(fPullDownMenuList);
	for (FW_CPullDownMenu* pullDownMenu = (FW_CPullDownMenu*)ite.First(); ite.IsNotComplete(); pullDownMenu = (FW_CPullDownMenu*)ite.Next())
	{
		if (enable)
			::EnableItem(GetMenuHandle(pullDownMenu->GetMenuID(ev)), 0);
		else
			::DisableItem(GetMenuHandle(pullDownMenu->GetMenuID(ev)), 0);
	}
	
	::InvalMenuBar();
}
#endif

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::EnableCommand
//----------------------------------------------------------------------------------------

void FW_CMenuBar::EnableCommand(Environment* ev, ODCommandID cmdNumber, FW_Boolean enable)
{
	fODMenuBar->EnableCommand(ev, cmdNumber, enable);

#ifdef FW_BUILD_MAC
	PrivMacEnableParentItem(ev, cmdNumber);
#endif
}

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	FW_CMenuBar::PrivMacEnableParentItem
//----------------------------------------------------------------------------------------

void FW_CMenuBar::PrivMacEnableParentItem(Environment* ev, ODCommandID cmdNumber)
{
	ODMenuID menuID;
	ODMenuItemID menuItem;
	
	fODMenuBar->GetMenuAndItem(ev, cmdNumber, &menuID, &menuItem);
	
	FW_CPullDownMenu* menu = this->PrivFindMenuWithID(ev, menuID);
	if (menu == NULL)
		return;
		
	FW_CMenuItem *parentItem = menu->GetParentMenuItem(ev);
	if (parentItem == NULL)
		return;
		
	ODPlatformMenu platformMenu = menu->GetPlatformMenu(ev);
	long flags = (*platformMenu)->enableFlags;
	
	if ((flags & 0xFFFFFFFE) != 0)			// if any items enabled
		flags |= 0x00000001;				// Enabled top level item
	else
		flags &= 0x11111110;				// Disabled top level item
	
	if ((flags & 0x00000001) != ((*platformMenu)->enableFlags & 0x00000001))
	{
		(*platformMenu)->enableFlags = flags;
	}
	
	ODPlatformMenu parentMenu = parentItem->GetOwnerMenu(ev)->GetPlatformMenu(ev);
	if ((flags & 0xFFFFFFFE) != 0)	// if any items enabled
		::EnableItem(parentMenu, parentItem->GetIndex(ev));
	else
		::DisableItem(parentMenu, parentItem->GetIndex(ev));
}
#endif

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::GetMenuItemFromCommand
//----------------------------------------------------------------------------------------

FW_CMenuItem* FW_CMenuBar::GetMenuItemFromCommand(Environment* ev, ODCommandID cmdNumber) const
{
	// ----- Look the top level first -----
	FW_COrderedCollectionIterator ite(fPullDownMenuList);
	for (FW_CPullDownMenu* pullDownMenu = (FW_CPullDownMenu*)ite.First(); ite.IsNotComplete(); pullDownMenu = (FW_CPullDownMenu*)ite.Next())
	{
		FW_CMenuItem* menuItem = pullDownMenu->PrivGetMenuItem(ev, cmdNumber);
		if (menuItem != NULL)
			return menuItem;
	}	
	return NULL;	
/*
#ifdef FW_BUILD_MAC
	ODMenuID menuID;
	ODMenuItemID menuItem;
	
	fODMenuBar->GetMenuAndItem(ev, cmdNumber, &menuID, &menuItem);
	FW_CPullDownMenu* menu = this->PrivFindMenuWithID(ev, menuID);
	if (menu == NULL)
		return NULL;

	return menu->PrivGetMenuItem(ev, cmdNumber);
#endif
*/
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::ToggleItem
//----------------------------------------------------------------------------------------

void FW_CMenuBar::ToggleItem(Environment* ev, 
							ODCommandID cmdNumber, 
							FW_Boolean toggleState)
{
	FW_CMenuItem* menuItem = GetMenuItemFromCommand(ev, cmdNumber);
	FW_ASSERT(menuItem != NULL);
	
#ifdef FW_DEBUG
	FW_CToggleItem* toggleItem = FW_DYNAMIC_CAST(FW_CToggleItem, menuItem);
	FW_ASSERT(toggleItem);
	toggleItem->ToggleItem(ev, toggleState);
#else
	((FW_CToggleItem*)menuItem)->ToggleItem(ev, toggleState);
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::DisableAll
//----------------------------------------------------------------------------------------

void FW_CMenuBar::DisableAll(Environment* ev)
{
	FW_COrderedCollectionIterator ite(fPullDownMenuList);
	for (FW_CPullDownMenu* pullDownMenu = (FW_CPullDownMenu*)ite.First(); ite.IsNotComplete(); pullDownMenu = (FW_CPullDownMenu*)ite.Next())
	{
		pullDownMenu->DisableAll(ev);
	}	
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::EnableAll
//----------------------------------------------------------------------------------------

void FW_CMenuBar::EnableAll(Environment* ev)
{
	FW_COrderedCollectionIterator ite(fPullDownMenuList);
	for (FW_CPullDownMenu* pullDownMenu = (FW_CPullDownMenu*)ite.First(); ite.IsNotComplete(); pullDownMenu = (FW_CPullDownMenu*)ite.Next())
	{
		pullDownMenu->EnableAll(ev);
	}	
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::CheckCommand
//----------------------------------------------------------------------------------------

void FW_CMenuBar::CheckCommand(Environment* ev, ODCommandID cmdNumber, FW_Boolean check)
{
	fODMenuBar->CheckCommand(ev, cmdNumber, check);
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::EnableAndCheckCommand
//----------------------------------------------------------------------------------------

void FW_CMenuBar::EnableAndCheckCommand(Environment* ev, ODCommandID cmdNumber, FW_Boolean enable, FW_Boolean check)
{
	fODMenuBar->EnableAndCheckCommand(ev, cmdNumber, enable, check);

#ifdef FW_BUILD_MAC
	PrivMacEnableParentItem(ev, cmdNumber);
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::EnableAndToggleCommand
//----------------------------------------------------------------------------------------

void FW_CMenuBar::EnableAndToggleCommand(Environment* ev, ODCommandID cmdNumber, FW_Boolean enable, FW_Boolean toggleState)
{
	EnableCommand(ev, cmdNumber, enable);
	ToggleItem(ev, cmdNumber, toggleState);
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::SetItemString
//----------------------------------------------------------------------------------------

void FW_CMenuBar::SetItemString(Environment* ev, ODCommandID commandID, const FW_CString& itemString)
{
	ODIText* intlText = CreateITextString(itemString);
	fODMenuBar->SetItemString(ev, commandID, intlText);
	::DisposeIText(intlText);
}

//----------------------------------------------------------------------------------------
//	FW_CMenuBar::GetItemString
//----------------------------------------------------------------------------------------

void FW_CMenuBar::GetItemString(Environment* ev, ODCommandID commandID, FW_CString& itemString) const
{
	ODIText* intlText = CreateIText(255);
	fODMenuBar->GetItemString(ev, commandID, intlText);
	FW_CIntlString iString(intlText);
	itemString = iString;
	::DisposeIText(intlText);
}
