//========================================================================================
//
//	File:				FWMnuItm.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWOS.hpp"

#ifndef FWMNUITM_H
#include "FWMnuItm.h"
#endif

#ifndef FWMENU_K
#include "FWMenu.k"
#endif

#ifndef FWPULLDM_H
#include "FWPullDM.h"
#endif

#ifndef FWMNUBAR_H
#include "FWMnuBar.h"
#endif

#ifndef FWINTSTR_H
#include "FWIntStr.h"
#endif

// ----- Foundation Includes -----

#ifndef FWSTREAM_H
#include "FWStream.h"
#endif

#ifndef FWSTRING_H
#include "FWString.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_ODMenuBar_xh
#include <MenuBar.xh>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__SCRIPT__)
#include <Script.h>
#endif

//========================================================================================
//	RunTime Info
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifdef FW_BUILD_MAC
#pragma segment fwmenu
#endif

FW_DEFINE_CLASS_M0(FW_CMenuItem)
FW_DEFINE_CLASS_M1(FW_CSeparatorItem, FW_CMenuItem)
FW_DEFINE_CLASS_M1(FW_CTextItem, FW_CMenuItem)
FW_DEFINE_CLASS_M1(FW_CToggleItem, FW_CTextItem)
FW_DEFINE_CLASS_M1(FW_CSubMenuItem, FW_CMenuItem)

FW_REGISTER_ARCHIVABLE_CLASS(FW_LMenuItem, FW_CMenuItem, FW_CMenuItem::Read, FW_CMenuItem::Write)
FW_REGISTER_ARCHIVABLE_CLASS(FW_LSeparatorItem, FW_CSeparatorItem, FW_CSeparatorItem::Read, FW_CMenuItem::Write)
FW_REGISTER_ARCHIVABLE_CLASS(FW_LTextItem, FW_CTextItem, FW_CTextItem::Read, FW_CMenuItem::Write)
FW_REGISTER_ARCHIVABLE_CLASS(FW_LToggleItem, FW_CToggleItem, FW_CToggleItem::Read, FW_CMenuItem::Write)
FW_REGISTER_ARCHIVABLE_CLASS(FW_LSubMenuItem, FW_CSubMenuItem, FW_CSubMenuItem::Read, FW_CMenuItem::Write)

//========================================================================================
//	class FW_CMenuItem
//========================================================================================

/*
//----------------------------------------------------------------------------------------
//	FW_CMenuItem::FW_CMenuItem
//----------------------------------------------------------------------------------------

FW_CMenuItem::FW_CMenuItem() :
	fOwnerMenu(NULL),
	fIndex(0)
#ifdef FW_BUILD_MAC
	,fCommandID(FW_kNoCommand)
#endif
{
}
*/

//----------------------------------------------------------------------------------------
//	FW_CMenuItem::FW_CMenuItem
//----------------------------------------------------------------------------------------

FW_CMenuItem::FW_CMenuItem(Environment* ev, FW_CReadableStream& archive) :
	fOwnerMenu(NULL)
#ifdef FW_BUILD_MAC
	,fCommandID(FW_kNoCommand)
#endif
{
	FW_READ_DYNAMIC_OBJECT(archive, &fOwnerMenu, FW_CPullDownMenu);
	
	archive >> fIndex;
}

//----------------------------------------------------------------------------------------
//	FW_CMenuItem::FW_CMenuItem
//----------------------------------------------------------------------------------------

FW_CMenuItem::FW_CMenuItem(Environment* ev, FW_CPullDownMenu* ownerMenu, short index) :
	fOwnerMenu(ownerMenu), 
	fIndex(index)
#ifdef FW_BUILD_MAC
	,fCommandID(FW_kNoCommand)
#endif
{
	FW_ASSERT(ownerMenu != NULL);
	FW_ASSERT(index > 0);
}

//----------------------------------------------------------------------------------------
//	FW_CMenuItem::~FW_CMenuItem
//----------------------------------------------------------------------------------------

FW_CMenuItem::~FW_CMenuItem()
{
}

//----------------------------------------------------------------------------------------
//	FW_CMenuItem::GetItemText
//----------------------------------------------------------------------------------------

void FW_CMenuItem::GetItemText(Environment* ev, FW_CString& itemText) const
{
	itemText = "";	
}

//----------------------------------------------------------------------------------------
//	FW_CMenuItem::GetCommandID
//----------------------------------------------------------------------------------------

ODCommandID FW_CMenuItem::GetCommandID(Environment* ev) const
{
	FW_ASSERT(fOwnerMenu != NULL);

#ifdef FW_BUILD_MAC
	return fCommandID;
#endif
	
#ifdef FW_BUILD_WIN
	return fOwnerMenu->GetCommandID(ev, fIndex);
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CMenuItem::PrivGetMenuItem
//----------------------------------------------------------------------------------------

FW_CMenuItem* FW_CMenuItem::PrivGetMenuItem(Environment* ev, ODCommandID commandID) const
{
	return NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CMenuItem::GetMenuKey
//----------------------------------------------------------------------------------------

FW_MenuKey FW_CMenuItem::GetMenuKey(Environment* ev) const
{
	return FW_kNoKeyEquivalent;
}

//----------------------------------------------------------------------------------------
//	FW_CMenuItem::PrivFindMenuWithID
//----------------------------------------------------------------------------------------

FW_CPullDownMenu* FW_CMenuItem::PrivFindMenuWithID(Environment* ev, ODMenuID menuID) const
{
	return NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CMenuItem::PrivDisableAll
//----------------------------------------------------------------------------------------

void FW_CMenuItem::PrivDisableAll(Environment* ev)
{
	// Nothing to do on the Mac because FW_CPullDownMenu::DisableAll disable all its items
	// at once
	
#ifdef FW_BUILD_WIN
	HMENU hMenu = fOwnerMenu->GetPlatformMenu(ev).menu;
	::EnableMenuItem(hMenu, fIndex - 1, MF_BYPOSITION | MF_DISABLED | MF_GRAYED);
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CMenuItem::PrivEnableAll
//----------------------------------------------------------------------------------------

void FW_CMenuItem::PrivEnableAll(Environment* ev)
{
	// Nothing to do on the Mac because FW_CPullDownMenu::DisableAll disable all its items
	// at once
	
#ifdef FW_BUILD_WIN
	HMENU hMenu = fOwnerMenu->GetPlatformMenu(ev).menu;
	::EnableMenuItem(hMenu, fIndex - 1, MF_BYPOSITION | MF_ENABLED);
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CMenuItem::PrivAttach
//----------------------------------------------------------------------------------------

void FW_CMenuItem::PrivAttach(Environment* ev, FW_CMenuBar* menuBar)
{
	// Nothing to do 
}

//----------------------------------------------------------------------------------------
//	FW_CMenuItem::PrivDetach
//----------------------------------------------------------------------------------------

void FW_CMenuItem::PrivDetach(Environment* ev, FW_CMenuBar* menuBar)
{
	// Nothing to do 
}

//----------------------------------------------------------------------------------------
//	FW_CMenuItem::Flatten
//----------------------------------------------------------------------------------------

void FW_CMenuItem::Flatten(Environment* ev, FW_CWritableStream& archive)
{
	FW_WRITE_DYNAMIC_OBJECT(archive, fOwnerMenu, FW_CPullDownMenu);
	
	archive << fIndex;
}

//----------------------------------------------------------------------------------------
//	FW_CMenuItem::Read
//----------------------------------------------------------------------------------------

void* FW_CMenuItem::Read(FW_CReadableStream& archive)
{
FW_UNUSED(archive);

	FW_DEBUG_MESSAGE("FW_CMenuItem::Read should never have been called");
	return NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CMenuItem::Write
//----------------------------------------------------------------------------------------

void FW_CMenuItem::Write(FW_CWritableStream& archive, const void* theMenuItem)
{
	Environment* ev = somGetGlobalEnvironment();
	
	FW_CMenuItem* menuItem = (FW_CMenuItem*)theMenuItem;
	menuItem->Flatten(ev, archive);
}

//========================================================================================
//	class FW_CSeparatorItem
//========================================================================================

/*
//----------------------------------------------------------------------------------------
//	FW_CSeparatorItem::FW_CSeparatorItem
//----------------------------------------------------------------------------------------

FW_CSeparatorItem::FW_CSeparatorItem() :
	FW_CMenuItem()
{
}
*/

//----------------------------------------------------------------------------------------
//	FW_CSeparatorItem::FW_CSeparatorItem
//----------------------------------------------------------------------------------------

FW_CSeparatorItem::FW_CSeparatorItem(Environment* ev, FW_CReadableStream& archive) :
	FW_CMenuItem(ev, archive)
{	
	InitSeparator(ev, fOwnerMenu->GetPlatformMenu(ev));
}

//----------------------------------------------------------------------------------------
//	FW_CSeparatorItem::FW_CSeparatorItem
//----------------------------------------------------------------------------------------

FW_CSeparatorItem::FW_CSeparatorItem(Environment* ev, FW_CPullDownMenu* ownerMenu, short index) :
	FW_CMenuItem(ev, ownerMenu, index)
{
	InitSeparator(ev, ownerMenu->GetPlatformMenu(ev));
}

//----------------------------------------------------------------------------------------
//	FW_CSeparatorItem::~FW_CSeparatorItem
//----------------------------------------------------------------------------------------

FW_CSeparatorItem::~FW_CSeparatorItem()
{
}

//----------------------------------------------------------------------------------------
//	FW_CSeparatorItem::InitSeparator
//----------------------------------------------------------------------------------------

void FW_CSeparatorItem::InitSeparator(Environment* ev, const ODPlatformMenu& platformMenu)
{
#ifdef FW_BUILD_MAC
	fCommandID = FW_kSeparatorCommand;
	::InsertMenuItem(platformMenu, "\p-", fIndex - 1);
#endif

#ifdef FW_BUILD_WIN
	::InsertMenu(platformMenu.menu, fIndex, MF_BYPOSITION | MF_SEPARATOR, 0, NULL);
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CSeparatorItem::PrivDisableAll
//----------------------------------------------------------------------------------------

void FW_CSeparatorItem::PrivDisableAll(Environment* ev)
{
	// Nothing to do. Already disable
}

//----------------------------------------------------------------------------------------
//	FW_CSeparatorItem::PrivEnableAll
//----------------------------------------------------------------------------------------

void FW_CSeparatorItem::PrivEnableAll(Environment* ev)
{
	// Should never enable it
}

//----------------------------------------------------------------------------------------
//	FW_CSeparatorItem::Read
//----------------------------------------------------------------------------------------

void* FW_CSeparatorItem::Read(FW_CReadableStream& archive)
{
	return new FW_CSeparatorItem(somGetGlobalEnvironment(), archive);
}

//========================================================================================
//	class FW_CTextItem
//========================================================================================

/*
//----------------------------------------------------------------------------------------
//	FW_CTextItem::FW_CTextItem
//----------------------------------------------------------------------------------------

FW_CTextItem::FW_CTextItem() :
	FW_CMenuItem()
{
}
*/

//----------------------------------------------------------------------------------------
//	FW_CTextItem::FW_CTextItem
//----------------------------------------------------------------------------------------

FW_CTextItem::FW_CTextItem(Environment* ev, FW_CReadableStream& archive) :
	FW_CMenuItem(ev, archive)
{
	ODCommandID commandID;
	archive >> commandID;
	
	FW_CDynamicString text;
	archive >> text;
	
	FW_MenuKey menuKey;
	archive >> menuKey;
	
	PrivInitTextItem(ev, fOwnerMenu->GetPlatformMenu(ev), text, commandID, menuKey);
}

//----------------------------------------------------------------------------------------
//	FW_CTextItem::FW_CTextItem
//----------------------------------------------------------------------------------------

FW_CTextItem::FW_CTextItem(Environment* ev,
						FW_CPullDownMenu* ownerMenu, 
						short index,			
						const FW_CString& text,
						ODCommandID commandID,
						FW_MenuKey menuKey) :
	FW_CMenuItem(ev, ownerMenu, index)
{
	FW_ASSERT(commandID > 0);
	PrivInitTextItem(ev, ownerMenu->GetPlatformMenu(ev), text, commandID, menuKey);
}

//----------------------------------------------------------------------------------------
//	FW_CTextItem::~FW_CTextItem
//----------------------------------------------------------------------------------------

FW_CTextItem::~FW_CTextItem()
{
}

//----------------------------------------------------------------------------------------
//	FW_CTextItem::PrivInitTextItem
//----------------------------------------------------------------------------------------

void FW_CTextItem::PrivInitTextItem(Environment* ev,
							  const ODPlatformMenu&	platformMenu,
							  const FW_CString& text,
							  ODCommandID commandID,
							  FW_MenuKey menuKey)
{
	FW_ASSERT(commandID >= FW_kFirstUserCommandID);
	
#ifdef FW_BUILD_MAC
	fCommandID = commandID;
	
	Str255 str;
	text.ExportPascal(str);
	::InsertMenuItem(platformMenu, "\pabc", fIndex - 1);
	::SetMenuItemText(platformMenu, fIndex, str);
	
	FW_CIntlString* intlText = FW_DYNAMIC_CAST(FW_CIntlString, &text);
	if (intlText)
	{
		ODScriptCode systemScriptCode = (ODScriptCode) ::GetScriptManagerVariable(smSysScript);
		if (intlText->GetScriptCode() != systemScriptCode)
		{	// set this menu item's script code
			::SetItemCmd(platformMenu, fIndex, FW_kScriptCodeKeyEquivalent);
			::SetItemIcon(platformMenu, fIndex, (short) intlText->GetScriptCode());
			menuKey = FW_kNoKeyEquivalent;	// so that code below doesn't mess up the key field we just set
		}
	}
	
	if (menuKey != FW_kNoKeyEquivalent)
		::SetItemCmd(platformMenu, fIndex, menuKey & FW_kMenuKeyCharMask);
#endif

#ifdef FW_BUILD_WIN
	::InsertMenu(platformMenu.menu, fIndex, MF_BYPOSITION | MF_STRING | MF_DISABLED | MF_UNCHECKED, commandID, text);
	//::AppendMenu(platformMenu.menu, MF_STRING | MF_DISABLED | MF_UNCHECKED, commandID, text);
	
	// [HLX] No way yet to build accelerator table
FW_UNUSED(menuKey);
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CTextItem::GetItemText
//----------------------------------------------------------------------------------------

void FW_CTextItem::GetItemText(Environment* ev, FW_CString& itemText) const
{
	const ODPlatformMenu& platformMenu = fOwnerMenu->GetPlatformMenu(ev);

#ifdef FW_BUILD_MAC
	Str255 str;
	::GetMenuItemText(platformMenu, fIndex, str);

	FW_CIntlString* intlText = FW_DYNAMIC_CAST(FW_CIntlString, &itemText);
	if (intlText)
	{	
		// Retrieve this menu item's script information
		short itemField;
		ODScriptCode scriptCode = (ODScriptCode) ::GetScriptManagerVariable(smSysScript);
		ODLangCode languageCode = (ODLangCode) ::GetScriptVariable(scriptCode, smScriptLang);
		::GetItemCmd(platformMenu, fIndex, &itemField);

		if (itemField == FW_kScriptCodeKeyEquivalent)	// item contains a script code
		{
			::GetItemIcon(platformMenu, fIndex, &itemField);
			if (itemField != 0x00FF)	/* kludge - $00FF is really 255, system script */
				scriptCode = (ODScriptCode) itemField;
			languageCode = (ODLangCode) ::GetScriptVariable(scriptCode, smScriptLang);

			FW_CIntlString intlString((const FW_Char*) &str[1], 
									  (FW_CharacterCount) str[0],
									  scriptCode, languageCode);
			itemText = intlString;
			return;
		}
	}

	// default - just copy characters from menu string to itemText
	itemText.ReplaceAll(str);
#endif

#ifdef FW_BUILD_WIN
	char str[256];
	::GetMenuString(platformMenu.menu, fIndex - 1, str, sizeof(str) - 1, MF_BYPOSITION);
	itemText = str;
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CTextItem::SetItemText
//----------------------------------------------------------------------------------------

void FW_CTextItem::SetItemText(Environment* ev, const FW_CString& itemText) const
{
	const ODPlatformMenu& platformMenu = fOwnerMenu->GetPlatformMenu(ev);

#ifdef FW_BUILD_MAC
	Str255 str;
	itemText.ExportPascal(str);
	::SetMenuItemText(platformMenu, fIndex, str);

	FW_CIntlString* intlText = FW_DYNAMIC_CAST(FW_CIntlString, &itemText);
	if (intlText)
	{
		ODScriptCode systemScriptCode = (ODScriptCode) ::GetScriptManagerVariable(smSysScript);
		if (intlText->GetScriptCode() != systemScriptCode)
		{	
			// set this menu item's script code
			::SetItemCmd(platformMenu, fIndex, FW_kScriptCodeKeyEquivalent);
			::SetItemIcon(platformMenu, fIndex, (short) intlText->GetScriptCode());
		}
		else	// remove script code leftover from previous tenant
		{
			::SetItemCmd(platformMenu, fIndex, FW_kNoKeyEquivalent);
			::SetItemIcon(platformMenu, fIndex, 0);
		}
	}
#endif

#ifdef FW_BUILD_WIN
	char str[256];
	itemText.Export(str);
	::ModifyMenu(platformMenu.menu, fIndex - 1, MF_BYPOSITION | MF_STRING, GetCommandID(ev), str);
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CTextItem::GetMenuKey
//----------------------------------------------------------------------------------------

FW_MenuKey FW_CTextItem::GetMenuKey(Environment* ev) const
{
#ifdef FW_BUILD_MAC
	FW_MenuKey menuKey;
	::GetItemCmd(fOwnerMenu->GetPlatformMenu(ev), fIndex, &menuKey);
	return menuKey;
#endif

#ifdef FW_BUILD_WIN
	return 0;					// [HLX] no way yet to parse accelerator table
#endif
}

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	FW_CTextItem::PrivAttach
//----------------------------------------------------------------------------------------

void FW_CTextItem::PrivAttach(Environment* ev, FW_CMenuBar* menuBar)
{
	menuBar->PrivMacRegisterCommand(ev, fCommandID, fOwnerMenu->GetMenuID(ev), fIndex);
}
#endif

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	FW_CTextItem::PrivDetach
//----------------------------------------------------------------------------------------

void FW_CTextItem::PrivDetach(Environment* ev, FW_CMenuBar* menuBar)
{
	menuBar->PrivMacUnregisterCommand(ev, fCommandID);
}
#endif

//----------------------------------------------------------------------------------------
//	FW_CTextItem::Read
//----------------------------------------------------------------------------------------

void* FW_CTextItem::Read(FW_CReadableStream& archive)
{
FW_UNUSED(archive);

	return new FW_CTextItem(somGetGlobalEnvironment(), archive);
}

//----------------------------------------------------------------------------------------
//	FW_CTextItem::Flatten
//----------------------------------------------------------------------------------------

void FW_CTextItem::Flatten(Environment* ev, FW_CWritableStream& archive)
{
	FW_CMenuItem::Flatten(ev, archive);
		
	ODCommandID commandID = GetCommandID(ev);
	archive << commandID;

	FW_CDynamicString text;
	GetItemText(ev, text);
	archive << text;
		
	FW_MenuKey menuKey = GetMenuKey(ev);
	archive << menuKey;
}

//========================================================================================
//	class FW_CToggleItem
//========================================================================================

/*
//----------------------------------------------------------------------------------------
//	FW_CToggleItem::FW_CToggleItem
//----------------------------------------------------------------------------------------

FW_CToggleItem::FW_CToggleItem() :
	FW_CTextItem(),
	fToggleState(TRUE)
{
}
*/

//----------------------------------------------------------------------------------------
//	FW_CToggleItem::FW_CToggleItem
//----------------------------------------------------------------------------------------

FW_CToggleItem::FW_CToggleItem(Environment* ev, FW_CReadableStream& archive) :
	FW_CTextItem(ev, archive)
{
	archive >> fToggleState;
	archive >> fOtherText;
}

//----------------------------------------------------------------------------------------
//	FW_CToggleItem::FW_CToggleItem
//----------------------------------------------------------------------------------------

FW_CToggleItem::FW_CToggleItem(Environment* ev,
						FW_CPullDownMenu* ownerMenu, 
						short index,			
						const FW_CString& trueText,
						const FW_CString& falseText,
						ODCommandID commandID,
						FW_MenuKey menuKey) :
	FW_CTextItem(ev, ownerMenu, index, trueText, commandID, menuKey)
{
	fToggleState = TRUE;
	fOtherText = falseText;
}

//----------------------------------------------------------------------------------------
//	FW_CToggleItem::~FW_CToggleItem
//----------------------------------------------------------------------------------------

FW_CToggleItem::~FW_CToggleItem()
{
}

//----------------------------------------------------------------------------------------
//	FW_CToggleItem::GetTrueText
//----------------------------------------------------------------------------------------

void FW_CToggleItem::GetTrueText(Environment* ev, FW_CString& trueText) const
{
	if (fToggleState)
		GetItemText(ev, trueText);
	else
		trueText = fOtherText;
}

//----------------------------------------------------------------------------------------
//	FW_CToggleItem::GetFalseText
//----------------------------------------------------------------------------------------

void FW_CToggleItem::GetFalseText(Environment* ev, FW_CString& falseText) const
{
	if (fToggleState)
		falseText = fOtherText;
	else
		GetItemText(ev, falseText);
}

//----------------------------------------------------------------------------------------
//	FW_CToggleItem::ToggleItem
//----------------------------------------------------------------------------------------

void FW_CToggleItem::ToggleItem(Environment* ev, FW_Boolean newState)
{
	if (newState == fToggleState)
		return;
		
	FW_CDynamicString currentText;
	GetItemText(ev, currentText);
		
	fToggleState = newState;
	
	SetItemText(ev, fOtherText);
	
	fOtherText = currentText;
}

//----------------------------------------------------------------------------------------
//	FW_CToggleItem::Read
//----------------------------------------------------------------------------------------

void* FW_CToggleItem::Read(FW_CReadableStream& archive)
{
FW_UNUSED(archive);

	return new FW_CToggleItem(somGetGlobalEnvironment(), archive);
}

//----------------------------------------------------------------------------------------
//	FW_CToggleItem::Flatten
//----------------------------------------------------------------------------------------

void FW_CToggleItem::Flatten(Environment* ev, FW_CWritableStream& archive)
{
	FW_CTextItem::Flatten(ev, archive);
	
	archive << fToggleState;
	archive << fOtherText;
}

//========================================================================================
//	class FW_CSubMenuItem
//========================================================================================

/*
//----------------------------------------------------------------------------------------
//	FW_CSubMenuItem::FW_CSubMenuItem
//----------------------------------------------------------------------------------------

FW_CSubMenuItem::FW_CSubMenuItem() :
	FW_CMenuItem(),
	fSubMenu(NULL)
{
}
*/

//----------------------------------------------------------------------------------------
//	FW_CSubMenuItem::FW_CSubMenuItem
//----------------------------------------------------------------------------------------

FW_CSubMenuItem::FW_CSubMenuItem(Environment* ev, FW_CReadableStream& archive) :
	FW_CMenuItem(ev, archive)
{
	FW_READ_DYNAMIC_OBJECT(archive, &fSubMenu, FW_CPullDownMenu);
	
	PrivInitSubMenu(ev, fOwnerMenu->GetPlatformMenu(ev));
}

//----------------------------------------------------------------------------------------
//	FW_CSubMenuItem::FW_CSubMenuItem
//----------------------------------------------------------------------------------------

FW_CSubMenuItem::FW_CSubMenuItem(Environment* ev,
				FW_CPullDownMenu* ownerMenu, 
				short index,
				FW_CPullDownMenu* adoptSubMenu):
	FW_CMenuItem(ev, ownerMenu, index),
	fSubMenu(adoptSubMenu)
{
	FW_ASSERT(adoptSubMenu != NULL);
	PrivInitSubMenu(ev, ownerMenu->GetPlatformMenu(ev));
}

//----------------------------------------------------------------------------------------
//	FW_CSubMenuItem::~FW_CSubMenuItem
//----------------------------------------------------------------------------------------

FW_CSubMenuItem::~FW_CSubMenuItem()
{
	delete fSubMenu;
	fSubMenu = NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CSubMenuItem::PrivInitSubMenu
//----------------------------------------------------------------------------------------

void FW_CSubMenuItem::PrivInitSubMenu(Environment* ev, const ODPlatformMenu& platformMenu)
{
	const ODPlatformMenu& subPlatformMenu = fSubMenu->GetPlatformMenu(ev);

#ifdef FW_BUILD_MAC
	fSubMenu->SetParentMenuItem(ev, this);
	
	// ----- Add it -----
	::InsertMenuItem(platformMenu, "\pabc", fIndex - 1);
		
	::HLock((Handle)subPlatformMenu);
	::SetMenuItemText(platformMenu, fIndex, (*subPlatformMenu)->menuData);
	::HUnlock((Handle)subPlatformMenu);
	
	// ----- Set the cmd key to hMenuCmd to specify sub menu
	::SetItemCmd(platformMenu, fIndex, hMenuCmd);		// has sub menu
	
	// ----- The menu id of the submenu is in the mark character
	// 	We don't set the item mark because we don't know yet the 
	//	ID of the sub menu. Will be done by PrivAttachedToMenuBar.
#endif

#ifdef FW_BUILD_WIN
	::InsertMenu(platformMenu.menu,
				 fIndex,
				 MF_BYPOSITION | MF_POPUP,
				 (UINT) subPlatformMenu.menu,
				 subPlatformMenu.strMenu);
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CSubMenuItem::PrivFindMenuWithID
//----------------------------------------------------------------------------------------

FW_CPullDownMenu* FW_CSubMenuItem::PrivFindMenuWithID(Environment* ev, ODMenuID menuID) const
{
	if (fSubMenu->GetMenuID(ev) == menuID)
		return fSubMenu;
	else 
		return fSubMenu->PrivFindMenuWithID(ev, menuID);
}

//----------------------------------------------------------------------------------------
//	FW_CSubMenuItem::PrivGetMenuItem
//----------------------------------------------------------------------------------------

FW_CMenuItem* FW_CSubMenuItem::PrivGetMenuItem(Environment* ev, ODCommandID commandID) const
{
	return fSubMenu->PrivGetMenuItem(ev, commandID);
}

//----------------------------------------------------------------------------------------
//	FW_CSubMenuItem::PrivDisableAll
//----------------------------------------------------------------------------------------

void FW_CSubMenuItem::PrivDisableAll(Environment* ev)
{
//	FW_CMenuItem::PrivDisableAll(ev);
	
	fSubMenu->DisableAll(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CSubMenuItem::PrivEnableAll
//----------------------------------------------------------------------------------------

void FW_CSubMenuItem::PrivEnableAll(Environment* ev)
{
	FW_CMenuItem::PrivEnableAll(ev);
	
	fSubMenu->EnableAll(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CSubMenuItem::PrivAttach
//----------------------------------------------------------------------------------------

void FW_CSubMenuItem::PrivAttach(Environment* ev, FW_CMenuBar* menuBar)
{
	short subMenuID = fSubMenu->PrivAcquireMenuID(ev, menuBar);
#ifdef FW_BUILD_MAC
	::SetItemMark(fOwnerMenu->GetPlatformMenu(ev), fIndex, (FW_Char)subMenuID);
#endif

	menuBar->PrivAddSubMenu(ev, subMenuID, fSubMenu->GetPlatformMenu(ev));
	
	fSubMenu->PrivAttach(ev, menuBar);
}

//----------------------------------------------------------------------------------------
//	FW_CSubMenuItem::PrivDetach
//----------------------------------------------------------------------------------------

void FW_CSubMenuItem::PrivDetach(Environment* ev, FW_CMenuBar* menuBar)
{
#ifdef FW_BUILD_MAC
	::SetItemMark(fOwnerMenu->GetPlatformMenu(ev), fIndex, (FW_Char)0xFF);
#endif
	menuBar->PrivRemoveMenu(ev, fSubMenu->GetMenuID(ev));

	fSubMenu->PrivRelinquishMenuID(ev);
		
	fSubMenu->PrivDetach(ev, menuBar);
}

//----------------------------------------------------------------------------------------
//	FW_CSubMenuItem::Read
//----------------------------------------------------------------------------------------

void* FW_CSubMenuItem::Read(FW_CReadableStream& archive)
{
FW_UNUSED(archive);

	return new FW_CMenuItem(somGetGlobalEnvironment(), archive);
}

//----------------------------------------------------------------------------------------
//	FW_CSubMenuItem::Flatten
//----------------------------------------------------------------------------------------

void FW_CSubMenuItem::Flatten(Environment* ev, FW_CWritableStream& archive)
{
	FW_CMenuItem::Flatten(ev, archive);
	
	FW_WRITE_DYNAMIC_OBJECT(archive, fSubMenu, FW_CPullDownMenu);
}
